%% Processing heart rate using SST method for AAS conference paper
% Carl Howard 12 August 2018


%% Heart Signal definitions and limitations
% say that the healthy, expected BPM is between 100 to 3000 BPM
% which means the period between beats is t_shift = 60/300 to 60/100 sec
BPM_low=100;
BPM_high=200;


%% Read in the audio file
[y,Fs]=audioread('heart_beat_filtered_8kHz.wav');

%% Band pass filter the data
% This will  band pass filter the date from 50 to 200 Hz
d = designfilt('bandpassiir','FilterOrder',20, ...
                'HalfPowerFrequency1',50,'HalfPowerFrequency2',200, ...
                'SampleRate',Fs);


% Uncomment the following line if you want to see the filter response
%fvt = fvtool(d,'Fs',Fs);
%x_Fs_44kHz_upto_200Hz=filtfilt(d,x);

% Just work with the first xx seconds of audio
% t_max=length(x_Fs_44kHz_upto_200Hz)/Fs;

y_BP_15_200Hz=filtfilt(d,y);


%% Downsample to 200 Hz
% Now resample to a lower frequency
% from the original 8 kHz down to about 200 Hz.
% The resampling has to be an integer downsample from 8 kHz,
% Hence 8000 / 40 = 200 Hz, which is okay.
fs200=200;
y_200Hz=resample(y_BP_15_200Hz,fs200,Fs);

% Calculate an associated time vector
t_fs200_Hz = [1:(length(y_200Hz))]/(fs200);



% Calculate the envelope using a Hilbert transform
figure
[up_envelope,~] = envelope(y_200Hz*10,5,'peak');
plot(t_fs200_Hz,y_200Hz*10,t_fs200_Hz,up_envelope);

xlabel('Time (s)');
ylabel('Audio')

%%% Only calculate 5 seconds initially to see if the SST is working properly
%up_envelope2=up_envelope(1:fs200*5);

%% Calculate Wavelet SST
[sst2,f2] = wsst(up_envelope,fs200,'VoicesPerOctave',48,'ExtendSignal',true);
% Can use the 'bump' wavelet instead... but did not seem to improve results
% very much, so just use the default, which is the 'amor' wavelet.
%[sst2,f2] = wsst(up_envelope,fs200,'VoicesPerOctave',48,'ExtendSignal',true,'bump');

% Find the index in the frequency vector that has the highest heart rate of
% interest.
f_index_max=max(find(f2<(BPM_high/60)));

% Plot the sonogram up to the frequency / heart rate of interest
figure
hp = pcolor(t_fs200_Hz,f2(1:f_index_max)*60,abs(sst2(1:f_index_max,:)));
%hp = pcolor(t_fs200_Hz(1:fs200*5),f2(1:f_index_max)*60,abs(sst2(1:f_index_max,:)));
hp.EdgeColor = 'none';
xlabel('Time (s)');
ylabel('Heart Rate (BPM)')


%%
% Extract the line graph for heart rate vs time
% from the SST sonogram
figure
[ignore_me,SST_index2]=max(abs(sst2(1:f_index_max,:)));
heart_rate_from_SST2=f2(SST_index2)*60;
plot(t_fs200_Hz,heart_rate_from_SST2);
xlabel('Time (s)');
ylabel('Heart Rate (BPM)');
title('Heart Rate versus Time');


%% This is the data of manually counting the pulses from the envelope and the raw time size
% the first column is the time where the peak of S1 sound occurs,
% the second column is the BPM, which is calculated as 60/(t_{i}-t_{i-1})
manual_hr2=[0.785	151.8987342
1.17	155.8441558
1.56	153.8461538
1.985	141.1764706
2.405	142.8571429
2.835	139.5348837
3.245	146.3414634
3.62	160
4.02	150
4.4	157.8947368
4.775	160
5.165	153.8461538
5.565	150
5.965	150
6.38	144.5783133
6.795	144.5783133
7.205	146.3414634
7.625	142.8571429
8.005	157.8947368
8.4	151.8987342
8.805	148.1481481
9.215	146.3414634
9.61	151.8987342
10.02	146.3414634
10.44	142.8571429
10.87	139.5348837
11.29	142.8571429
11.71	142.8571429
12.12	146.3414634
12.53	146.3414634
12.9	162.1621622
13.3	150
13.71	146.3414634
14.13	142.8571429
14.54	146.3414634
14.96	142.8571429
15.39	139.5348837
15.83	136.3636364
16.25	142.8571429
16.64	153.8461538
17.05	146.3414634
17.46	146.3414634
17.87	146.3414634
18.27	150
18.69	142.8571429
19.13	136.3636364
19.54	146.3414634
19.98	136.3636364
20.42	136.3636364
];

%% Overplot the curve from the manual counting
hold on
pz1=plot(manual_hr2(:,1),manual_hr2(:,2),'--');
legend('SST','Manual counting')
axis([0 20 BPM_low BPM_high])





%% Calculate the Fourier SST
%[sst3,f3,t3] = fsst(up_envelope,fs200);

%f_index_max3=max(find(f3<(BPM_high/60)));

% Stop right here...
% The trouble is that when using the envelope signal resampled to 200Hz, 
% the frequency resolution is 46 Hz.
%          0
%    46.8750
%    93.7500
%   140.6250
%   187.5000
% Which is useless... so don't bother.

% figure
% p_fsst=pcolor(t3,f3(1:f_index_max3)*60,abs(sst3(1:f_index_max3,:)));
% hp.EdgeColor = 'none';
% xlabel('Time (s)');
% ylabel('Heart Rate (BPM)');

