%% Processing heart rate using peaks in envelope for AAS conference paper
% Carl Howard 12 August 2018


%% Heart Signal definitions and limitations
% say that the healthy, expected BPM is between 100 to 200 BPM
% which means the period between beats is t_shift = 60/200 to 60/100 sec
BPM_low=100;
BPM_high=200;


%% Read in the audio file
[y,Fs]=audioread('heart_beat_filtered_8kHz.wav');

%% Band pass filter the data
% This will  band pass filter the date from 50 to 200 Hz
% d = designfilt('bandpassiir','FilterOrder',20, ...
%                 'HalfPowerFrequency1',50,'HalfPowerFrequency2',200, ...
%                 'SampleRate',Fs);
            
% This will  band pass filter the date from 50 to 100 Hz
d = designfilt('bandpassiir','FilterOrder',20, ...
                'HalfPowerFrequency1',50,'HalfPowerFrequency2',100, ...
                'SampleRate',Fs);
          
y_BP=filtfilt(d,y);

time=[1:length(y_BP)].'/Fs;


%%
% Try plotting the envelope using a Hilbert transform
% figure
gain_for_envelope = 10;     % a constant to multiply the audio signal
[up_envelope,~] = envelope((y_BP*gain_for_envelope).^2,4,'peak');

index=find(up_envelope<0);
up_envelope(index)=0;
p1=plot(   time,(y_BP*gain_for_envelope).^2,   ...
        time,up_envelope  );
legend('Filtered 50-100Hz','Envelope');
xlabel('Time (s)');
ylabel('Audio')
title('Envelope Signal')


% find the peaks in the envelope spectrum
%[pks,locs] = findpeaks(up_envelope);
% The MinPeakDistance = 0.2 means that there should be a distance = time
% between peaks of (1/BPM_high)*60;
minpeakdistance=(1/BPM_high)*60;
[pks,locs] = findpeaks(up_envelope,time,'MinPeakDistance',minpeakdistance);


% Find the peaks that are above a value of 0.1 --- this will have to be
% tweaked for each file that is analysed. I just determined this value by
% looking at the plot of the envelope vs time.
% Might not even need this... using he MinPeakDistance seems to be
% sufficient.
index=find(pks>0.1);
pks2=pks(index);
locs2=locs(index);

% Plot the peaks on top of the envelope time series, which should be S1
hold on
p2=plot(locs2,pks2,'ro');
xlabel('Time (s)')
ylabel('Envelope Signal')
title({'First and Second Heart Pulses','heart\_beat\_filtered\_8kHz.wav'});
legend('Rectified Signal','Envelope, Hilbert transform','Peaks above 0.1')

axis([0 20 0 2])

%% Calculate the BPM from the peaks which should be on S1

S1_period=locs2(2:end)-locs2(1:end-1);
S1_BPM=ones(size(S1_period))./S1_period*60;

figure
p3=plot(locs2(2:end),S1_BPM);
xlabel('Time (s)')
ylabel('BPM')
title({'BPM calculated using peaks of the envelope','heart\_beat\_filtered\_8kHz.wav'});



%% This is the data of manually counting the pulses from the envelope and the raw time size
% the first column is the time where the peak of S1 sound occurs,
% the second column is the BPM, which is calculated as 60/(t_{i}-t_{i-1})
manual_hr2=[0.785	151.8987342
1.17	155.8441558
1.56	153.8461538
1.985	141.1764706
2.405	142.8571429
2.835	139.5348837
3.245	146.3414634
3.62	160
4.02	150
4.4	157.8947368
4.775	160
5.165	153.8461538
5.565	150
5.965	150
6.38	144.5783133
6.795	144.5783133
7.205	146.3414634
7.625	142.8571429
8.005	157.8947368
8.4	151.8987342
8.805	148.1481481
9.215	146.3414634
9.61	151.8987342
10.02	146.3414634
10.44	142.8571429
10.87	139.5348837
11.29	142.8571429
11.71	142.8571429
12.12	146.3414634
12.53	146.3414634
12.9	162.1621622
13.3	150
13.71	146.3414634
14.13	142.8571429
14.54	146.3414634
14.96	142.8571429
15.39	139.5348837
15.83	136.3636364
16.25	142.8571429
16.64	153.8461538
17.05	146.3414634
17.46	146.3414634
17.87	146.3414634
18.27	150
18.69	142.8571429
19.13	136.3636364
19.54	146.3414634
19.98	136.3636364
20.42	136.3636364
];

hold on
pz1=plot(manual_hr2(:,1),manual_hr2(:,2),'--');
legend('Peak of Envelope','Manual counting')
axis([0 20 BPM_low BPM_high])
